﻿// ECMAScript 5 strict mode
"use strict";

assert2(cr, "cr namespace not created");
assert2(cr.plugins_, "cr.plugins_ not created");

/////////////////////////////////////
// Plugin class
cr.plugins_.Metro = function (runtime)
{
	this.runtime = runtime;
};

(function ()
{
    var pluginProto = cr.plugins_.Metro.prototype;
		
	/////////////////////////////////////
	// Object type class
	pluginProto.Type = function(plugin)
	{
		this.plugin = plugin;
		this.runtime = plugin.runtime;
	};

	var typeProto = pluginProto.Type.prototype;

	// called on startup for each object type
	typeProto.onCreate = function()
	{
	};

	/////////////////////////////////////
	// Instance class
	pluginProto.Instance = function(type)
	{
		this.type = type;
		this.runtime = type.runtime;
		
	};
	
	var instanceProto = pluginProto.Instance.prototype;

	// called whenever an instance is created
	instanceProto.onCreate = function()
	{
		// note the object is sealed after this call; ensure any properties you'll ever need are set on the object
	    this.pluginEnabled = (this.properties[0] === 0) ? true : false;

	    if (this.pluginEnabled)
	    {
            // Retrieve other property values set by the Game developer
	        this.ShowPreferences = (this.properties[1] === 0) ? true : false;
	        this.ShowPurchase = (this.properties[2] === 0) ? true : false;
	        this.ShowHelp = (this.properties[3] === 0) ? true : false;
	        this.ShowAbout = (this.properties[4] === 0) ? true : false;

            // Preferences page elements
	        this.BGMOptionEnabled   = this.ShowPreferences && ((this.properties[5] === 0) ? true : false);
	        this.SFXOptionEnabled   = this.ShowPreferences && ((this.properties[6] === 0) ? true : false);            
	        this.ResetButtonEnabled = this.ShowPreferences && ((this.properties[7] === 0) ? true : false);

            // In app purchase/trial related properties
	        this.IsLicenseTestMode = ((this.properties[8] === 0) ? true : false);
            
            // Sensor usage
	        this.bUseAccelerometer = ((this.properties[9]  === 0) ? true : false);
	        this.bUseGyrometer     = ((this.properties[10] === 0) ? true : false);
	        this.bUseInclinometer  = ((this.properties[11] === 0) ? true : false);
	        this.bUseLightSensor   = ((this.properties[12] === 0) ? true : false);
           
	        if (this.ShowPreferences && !this.BGMOptionEnabled && !this.SFXOptionEnabled && !this.ResetButtonEnabled) {
	            this.ShowPreferences = false;
	        }
	            
	        if (this.IsLicenseTestMode) {
	            console.log("In test mode for in app purchase/trial mode");

	            // Load the testing xml
	            this.currentApp = Windows.ApplicationModel.Store.CurrentAppSimulator;
	            this.licenseInformation = this.currentApp.licenseInformation;

	            Windows.ApplicationModel.Package.current.installedLocation.getFileAsync("In-App-Purchase-TestingOnly.xml").done(
                    function (file) {
                        Windows.ApplicationModel.Store.CurrentAppSimulator.reloadSimulatorAsync(file).done(
                            function (){
                                console.log("Successfully loaded in app purchase testing profile");
                            },
                            function (errMsg) {
                                console.log("Problem loading in app purchase testing profile" + errMsg);
                            })
                    },
                    function (errMsg) {
                        console.log("Error loading file In-App-Purchase-TestingOnly.xml" + errMsg);
                    });	                
	        }
	        else {
	            this.currentApp = Windows.ApplicationModel.Store.CurrentApp;
	            this.licenseInformation = this.currentApp.licenseInformation;
	        }	            


	        this.GameViewState = Windows.UI.ViewManagement.ApplicationViewState;

	        this.GameBackgroundMusicState = true;
	        this.GameSoundEffectsState = true;

	        var focusoutEvent = (function (self) {
	            return function (info) {
	                self.onfocusout(info);
	            };
	        })(this);

	        var beforeShowEvent = (function (self) {
	            return function (info) {
	                self.onbeforeshow(info);
	            };
	        })(this);

	        var OnResizeEvent = (function (self) {
	            return function (info) {
	                self.OnResize(info);
	            };
	        })(this);

	        var OnSettingsEvent = (function (self) {
	            return function (info) {
	                self.OnSettings(info);
	            };
	        })(this);

	        document.addEventListener("focusout", focusoutEvent, false);
	        document.addEventListener("beforeshow", beforeShowEvent, false);
	        window.addEventListener("resize", OnResizeEvent);
	        WinJS.Application.addEventListener("settings", OnSettingsEvent);


	        // Set up the Accelerometer
	        this.accelerometer = null;
	        this.accX = this.accY = this.accZ = 0;

	        var onAccDataChanged = (function (self) {
	            return function (e) {
	                var reading = e.reading;
	                self.accX = reading.accelerationX;
	                self.accY = reading.accelerationY;
	                self.accZ = reading.accelerationZ;
	            };
	        })(this);

	        var onAccShaken = (function (self) {
	            return function (e) {
	                // Trigger shaken event
	                self.runtime.trigger(cr.plugins_.Metro.prototype.cnds.OnShaken, self);
	            };
	        })(this);

	        if (this.bUseAccelerometer) {
	            this.accelerometer = Windows.Devices.Sensors.Accelerometer.getDefault();
	            if (this.accelerometer) {
	                var minimumReportInterval = this.accelerometer.minimumReportInterval;
	                this.accelerometer.reportInterval = minimumReportInterval > 16 ? minimumReportInterval : 16;
	            } else {
	                console.log("Accelerometer not available");
	            }
	        }


	        // Set up the Gyrometer
	        this.gyrometer = null;
	        this.angVelX = this.angVelY = this.angVelZ = 0;

	        var onGyroDataChanged = (function (self) {
	            return function (e) {
	                var reading = e.reading;
	                self.angVelX = reading.angularVelocityX;
	                self.angVelY = reading.angularVelocityY;
	                self.angVelZ = reading.angularVelocityZ;
	            };
	        })(this);

	        if (this.bUseGyrometer) {
	            this.gyrometer = Windows.Devices.Sensors.Gyrometer.getDefault();
	            if (this.gyrometer) {
	                var minimumReportInterval = this.gyrometer.minimumReportInterval;
	                this.gyrometer.reportInterval = minimumReportInterval > 16 ? minimumReportInterval : 16;
	            } else {
	                console.log("Gyrometer not available");
	            }
	        }

	        // Set up the Inclinometer
	        this.inclinometer = null;
	        this.pitch = this.roll = this.yaw = 0;

	        var onInclinoDataChanged = (function (self) {
	            return function (e) {
	                var reading = e.reading;
	                self.pitch = reading.pitchDegrees;
	                self.roll  = reading.rollDegrees;
	                self.yaw   = reading.yawDegrees;
	            };
	        })(this);

	        if (this.bUseInclinometer) {
	            this.inclinometer = Windows.Devices.Sensors.Inclinometer.getDefault();
	            if (this.inclinometer) {
	                var minimumReportInterval = this.inclinometer.minimumReportInterval;
	                this.inclinometer.reportInterval = minimumReportInterval > 16 ? minimumReportInterval : 16;
	            } else {
	                console.log("Inclinometer not available");
	            }
	        }

	        // Set up the LightSensor
	        this.lightSensor = null;
	        this.illuminanceInLux = 0;

	        var onLightSensorChanged = (function (self) {
	            return function (e) {
	                var reading = e.reading;
	                self.illuminanceInLux = reading.illuminanceInLux;	                
	            };
	        })(this);

	        if (this.bUseLightSensor) {
	            this.lightSensor = Windows.Devices.Sensors.LightSensor.getDefault();
	            if (this.lightSensor) {
	                var minimumReportInterval = this.lightSensor.minimumReportInterval;
	                this.lightSensor.reportInterval = minimumReportInterval > 16 ? minimumReportInterval : 16;

	            } else {
	                console.log("LightSensor not available");
	            }
	        }


	        var addEventListeners = function (self) {
	            if (self.bUseAccelerometer && self.accelerometer) {
	                self.accelerometer.addEventListener("readingchanged", onAccDataChanged);
	                self.accelerometer.addEventListener("shaken", onAccShaken);
	            }
	            if (self.bUseGyrometer && self.gyrometer) { self.gyrometer.addEventListener("readingchanged", onGyroDataChanged); }
	            if (self.bUseInclinometer && self.inclinometer) { self.inclinometer.addEventListener("readingchanged", onInclinoDataChanged); }
	            if (self.bUseLightSensor && self.lightSensor) { self.lightSensor.addEventListener("readingchanged", onLightSensorChanged); }
	        }

	        var visibilityChangeEvent = (function (self) {
	            return function () {
	                if (document.msVisibilityState === "visible") {
	                    addEventListeners(self);
	                } else {
	                    if (self.bUseAccelerometer && self.accelerometer) {
	                        self.accelerometer.removeEventListener("readingchanged", onAccDataChanged);
	                        self.accelerometer.removeEventListener("shaken", onAccShaken);
	                    }
	                    if (self.bUseGyrometer && self.gyrometer) { self.gyrometer.removeEventListener("readingchanged", onGyroDataChanged); }
	                    if (self.bUseInclinometer && self.inclinometer) { self.inclinometer.removeEventListener("readingchanged", onInclinoDataChanged); }
	                    if (self.bUseLightSensor && self.lightSensor) { self.lightSensor.removeEventListener("readingchanged", onLightSensorChanged); }
	                }
	            };
	        })(this);
	        document.addEventListener("visibilitychange", visibilityChangeEvent, false);

	        // Add the event listeners initially
	        addEventListeners(this);
	    }
    };
	
	// only called if a layout object - draw to a canvas 2D context
	instanceProto.draw = function(ctx)
	{
	};
	
	// only called if a layout object in WebGL mode - draw to the WebGL context
	// 'glw' is not a WebGL context, it's a wrapper - you can find its methods in GLWrap.js in the install
	// directory or just copy what other plugins do.
	instanceProto.drawGL = function (glw)
	{
	};

	instanceProto.OnResize = function (e)
	{
	    var myViewState = Windows.UI.ViewManagement.ApplicationView.value;
	    var viewStates  = Windows.UI.ViewManagement.ApplicationViewState;
	    
	    if (myViewState === viewStates.snapped ||
	        myViewState === viewStates.filled ||
	        myViewState === viewStates.fullScreenLandscape ||
	        myViewState === viewStates.fullScreenPortrait)
	    {
	        this.GameViewState = myViewState;
	        this.runtime.trigger(cr.plugins_.Metro.prototype.cnds.OnResize, this);
	    }
	};

	instanceProto.onfocusout = function ()
	{
	    this.runtime.trigger(cr.plugins_.Metro.prototype.cnds.OnGameLostFocus, this);
	};

	instanceProto.onToggleChange = function (e)
	{
	    if (e.srcElement.id === "toggleGameBGM") {
	        this.GameBackgroundMusicState = toggleGameBGM.winControl.checked;
	        this.runtime.trigger(cr.plugins_.Metro.prototype.cnds.OnGameBackgroundMusicChanged, this);
	    }
	    else if (e.srcElement.id === "toggleGameSounds") {
	        this.GameSoundEffectsState = toggleGameSounds.winControl.checked;
	        this.runtime.trigger(cr.plugins_.Metro.prototype.cnds.OnGameSoundEffectsChanged, this);
	    }
	};
	instanceProto.onResetGame = function () {
	    this.runtime.trigger(cr.plugins_.Metro.prototype.cnds.OnGameReset, this);
	}

	instanceProto.onbeforeshow = function (e)
	{
	    if (e.srcElement.id == "divGamePurchase") {
	        if (this.ShowPurchase) {
	            var showTrialUpgradeOption = function (divPurchase, self, buttonCaption) {
	                var btnPurchase = document.createElement('input');
	                btnPurchase.type = 'button';
	                btnPurchase.value = buttonCaption;

	                btnPurchase.onclick = (function (self, isTestMode, currApp) {
	                    return function () {
	                        if (!currApp.licenseInformation.isActive || currApp.licenseInformation.isTrial) {
	                            currApp.requestAppPurchaseAsync(false).done(
                                function () {
                                    if (currApp.licenseInformation.isActive && !currApp.licenseInformation.isTrial) {
                                        if (isTestMode) { console.log("Purchased the app"); }
                                        // Once the purchase was successful, invoke the callback
                                        self.runtime.trigger(cr.plugins_.Metro.prototype.cnds.OnPurchaseCompleted, self);
                                    } else {
                                        if (isTestMode) { console.log("Still in trial mode"); }
                                    }
                                },
                                function () {
                                    if (isTestMode) { console.log("Upgrade transaction failed. Still in trial mode"); }
                                });
	                        }

	                    };
	                })(self, self.IsLicenseTestMode, self.currentApp);
	                divPurchase.appendChild(btnPurchase);
	            }; // end:showTrialUpgradeOption

	            var showFeaturePurchaseOption = function (divPurchase, self, buttonCaption, featureId) {
	                var btnPurchase = document.createElement('input');
	                btnPurchase.type = 'button';
	                btnPurchase.value = buttonCaption;

	                btnPurchase.onclick = (function (self, isTestMode, currApp, featureId) {
	                    return function () {
	                        if (!currApp.licenseInformation.productLicenses.lookup(featureId).isActive) {

	                            currApp.requestProductPurchaseAsync(featureId, false).then(
                                    function () {
                                        if (currApp.licenseInformation.productLicenses.lookup(featureId).isActive) {
                                            // the in-app purchase was successful
                                            if (isTestMode) { console.log("Purchasing of feature " + featureId + " successful"); }
                                            // Once the purchase was successful, invoke the callback
                                            self.runtime.trigger(cr.plugins_.Metro.prototype.cnds.OnPurchaseCompleted, self);
                                        } else {
                                            // Something went wrong
                                            if (isTestMode) { console.log("Failed purchasing feature " + featureId + " after invoking self.currentApp.requestProductPurchaseAsync"); }
                                        }
                                    },
                                    function (errMsg) {
                                        // The in-app purchase was not completed because 
                                        // the customer canceled it or there was an error.
                                        if (isTestMode) { console.log("self.currentApp.requestProductPurchaseAsync returned failure" + errMsg); }
                                    });
	                        }
	                        else {
	                            // The customer already owns this feature.
	                            if (isTestMode) { console.log(featureId + " License already owned by user"); }
	                        }

	                    };
	                })(self, self.IsLicenseTestMode, self.currentApp, featureId);
	                divPurchase.appendChild(btnPurchase);
	            };//end: showFeaturePurchaseOption

                // This function is defined in Settings-Purchase.html shipping with this plugin
	            initPurchaseSettings(this, showTrialUpgradeOption, showFeaturePurchaseOption);
	        }
	    }
	    else if (e.srcElement.id === "divGamePreferences")
	    {
	        if (this.BGMOptionEnabled) {
	            var toggleBGMElement = document.getElementById("toggleGameBGM");
	            var toggleBGM = new WinJS.UI.ToggleSwitch(toggleBGMElement, { title: 'Background Music', checked: this.GameBackgroundMusicState });
	        }
	        if (this.SFXOptionEnabled) {
	            var toggleSoundsElement = document.getElementById("toggleGameSounds");
	            var toggleSounds = new WinJS.UI.ToggleSwitch(toggleSoundsElement, { title: 'Sound Effects', checked: this.GameSoundEffectsState });
	        }
	        if (this.ResetButtonEnabled) {
	            var divResetGame = document.getElementById("divResetGame");
	            divResetGame.innerHTML = "<h2> Reset </h2> <p> You will lose all the progress made in the game including unlocked levels and collected stars. </p>";
	            var btnReset = document.createElement('input');
	            btnReset.id = "btnReset";
	            btnReset.type = 'button';
	            btnReset.value = "Reset";
	            btnReset.onclick = (function (self) {
	                return function () {
	                    var divConfirmation = document.getElementById("divConfirmation");
	                    btnReset.disabled = true;
	                    divConfirmation.innerHTML = "<h3> Please Confirm </h3> <p> Are you sure you want to revert all the progress? </p>";

                        // Add Yes and No to the confirmation div
	                    var btnYes = document.createElement('input');
	                    btnYes.type = 'button';
	                    btnYes.value = "Yes";

	                    btnYes.onclick = (function (self) {
	                        return function () {
	                            // Clear the app data - local and roaming
	                            Windows.Storage.ApplicationData.current.clearAsync().done(
                                    function succcessHandler(parent) {
                                        // On success, trigger the C2 conditions
                                        //self.onResetGame();
                                        self.runtime.trigger(cr.plugins_.Metro.prototype.cnds.OnGameReset, self);
                                        document.getElementById("btnReset").disabled = false;
                                        document.getElementById("divConfirmation").innerHTML = "";
                                    },
                                    function failureHandler() {
                                        document.getElementById("btnReset").disabled = false;
                                        document.getElementById("divConfirmation").innerHTML = "";
                                    }
                                );
	                        };
	                    })(self); // make sure to pass in self param here (not this)

	                    divConfirmation.appendChild(btnYes);

	                    var btnNo = document.createElement('input');
	                    btnNo.type = 'button';
	                    btnNo.value = "No";

	                    btnNo.onclick = (function (self) {
	                        return function () {
	                            document.getElementById("btnReset").disabled = false;
	                            document.getElementById("divConfirmation").innerHTML = "";
	                        };
	                    })(self); // make sure to pass in self param here (not this)

	                    divConfirmation.appendChild(btnNo);
	                };
	            })(this);

	            divResetGame.appendChild(btnReset);
	        }
	        if (this.BGMOptionEnabled || this.SFXOptionEnabled)
	        {
	            var togglechangeEvent = (function (self) {
	                return function (info) {
	                    self.onToggleChange(info);
	                };
	            })(this);
	            if (this.BGMOptionEnabled) {
	                toggleBGMElement.addEventListener("change", togglechangeEvent);
	            }
	            if (this.SFXOptionEnabled) {
	                toggleSoundsElement.addEventListener("change", togglechangeEvent);
	            }
	        }
	    }
	};

	instanceProto.OnSettings = function (e)
	{
	    e.detail.applicationcommands = [];

	    if (this.ShowPreferences) { e.detail.applicationcommands["preferences"] = { title: "Preferences",   href: "Settings-GamePreferences.html" }; }
	    if (this.ShowPurchase)    { e.detail.applicationcommands["purchase"]    = { title: "Purchase",      href: "Settings-Purchase.html" }; }
	    if (this.ShowHelp)        { e.detail.applicationcommands["help"]        = { title: "Help",          href: "Settings-Help.html" }; }
	    if (this.ShowAbout)       { e.detail.applicationcommands["about"]       = { title: "About",        href: "Settings-About.html" };}

	    WinJS.UI.SettingsFlyout.populateSettings(e);
	};

	//////////////////////////////////////
	// Conditions
	function Cnds() {};

	Cnds.prototype.OnGameLostFocus = function ()
	{
	    return true;
	};	
	
	Cnds.prototype.OnResize = function ()
	{
	    return true;
	};

	Cnds.prototype.OnGameBackgroundMusicChanged = function ()
	{
	    return true;
	};

	Cnds.prototype.OnGameSoundEffectsChanged = function ()
	{
	    return true;
	};

	Cnds.prototype.OnGameReset = function () {
	    return true;
	};

	Cnds.prototype.OnPurchaseCompleted = function () {
	    return true;
	};
	
	Cnds.prototype.LocalKeyExists = function (key)
	{
	    if (this.pluginEnabled) {
	        var localSettings = Windows.Storage.ApplicationData.current.localSettings;
	        return localSettings.values.hasKey(key);
	    } else {
	        return false;
	    }
	};

	Cnds.prototype.RoamingKeyExists = function (key) {
	    if (this.pluginEnabled) {
	        var roamingSettings = Windows.Storage.ApplicationData.current.roamingSettings;
	        return roamingSettings.values.hasKey(key);
	    } else {
	        return false;
	    }
	};

	Cnds.prototype.IsInTrialMode = function () {
	    if (this.pluginEnabled
            && this.licenseInformation.isActive
            && this.licenseInformation.isTrial) {
	        return true;
	    } else {
	        return false;
	    }
	};
        
	Cnds.prototype.IsInAppFeaturePurchased = function (featureId) {
	    if (this.pluginEnabled
            && this.licenseInformation.productLicenses.lookup(featureId).isActive) {
	        return true;
	    } else {
	        return false;
	    }
	};

	Cnds.prototype.MetroEnabled = function () {
	    return this.pluginEnabled;
	};
	
	Cnds.prototype.IsAccelerometerAvailable = function () {
	    return this.pluginEnabled && this.bUseAccelerometer && (this.accelerometer != null);
	};

	Cnds.prototype.IsGyrometerAvailable = function () {
	    return this.pluginEnabled && this.bUseGyrometer && (this.gyrometer != null);
	};

	Cnds.prototype.IsInclinometerAvailable = function () {
	    return this.pluginEnabled && this.bUseInclinometer && (this.inclinometer != null);
	};

	Cnds.prototype.IsLightSensorAvailable = function () {
	    return this.pluginEnabled && this.bUseLightSensor && (this.lightSensor != null);
	};

	Cnds.prototype.OnShaken = function () {
	    return true;
	};

	pluginProto.cnds = new Cnds();
	
	//////////////////////////////////////
	// Actions
	function Acts() {};

	Acts.prototype.InvokeSettingsCharm = function ()
	{
	    if (this.pluginEnabled) {
	        WinJS.UI.SettingsFlyout.show();
	    }
	};

	Acts.prototype.InvokeSettingsPreferences = function () {
	    if (this.pluginEnabled && this.ShowPreferences) {
	        WinJS.UI.SettingsFlyout.showSettings("preferences", "Settings-GamePreferences.html");
	    }
	};

	Acts.prototype.InvokeSettingsPurchase = function () {
	    if (this.pluginEnabled && this.ShowPurchase) {        
	            WinJS.UI.SettingsFlyout.showSettings("purchase", "Settings-Purchase.html");
	    }
	};

	Acts.prototype.SetGameBackGroundMusic = function (idx) {
	    if (this.pluginEnabled) {
	        this.GameBackgroundMusicState = (idx === 0) ? true : false;
	    }
	};

	Acts.prototype.SetGameSoundEffects = function (idx) {
	    if (this.pluginEnabled) {
	        this.GameSoundEffectsState = (idx === 0) ? true : false;
	    }
	};
    
	Acts.prototype.RequestUnsnap = function () {
	    if (this.pluginEnabled) {
	        Windows.UI.ViewManagement.ApplicationView.tryUnsnap();
	    }
	};

	Acts.prototype.UpdateLiveTile = function (txtMessage) {

	    if (this.pluginEnabled) {
	        var Notifications = Windows.UI.Notifications;
	        var tileXml = Notifications.TileUpdateManager.getTemplateContent(Notifications.TileTemplateType.tileWideImageAndText01);

	        var tileAttributes = tileXml.getElementsByTagName("text");
	        tileAttributes[0].appendChild(tileXml.createTextNode(txtMessage));

	        var tileImageAttributes = tileXml.getElementsByTagName("image");
	        tileImageAttributes[0].setAttribute("src", "ms-appx:///app-widelogo.png");

	        var tileNotification = new Notifications.TileNotification(tileXml);
	        Notifications.TileUpdateManager.createTileUpdaterForApplication().update(tileNotification);
	    }
	};

	Acts.prototype.StoreLocal = function (key, data) {
	    if (this.pluginEnabled) {
	        var localSettings = Windows.Storage.ApplicationData.current.localSettings;
	        localSettings.values[key] = data;
	    }
	};

	Acts.prototype.RemoveLocal = function (key) {
	    if (this.pluginEnabled) {
	        var localSettings = Windows.Storage.ApplicationData.current.localSettings;
	        localSettings.values.remove(key);
	    }
	};

	Acts.prototype.ClearLocal = function () {
	    if (this.pluginEnabled) {
	        var localSettings = Windows.Storage.ApplicationData.current.localSettings;
	        localSettings.values.clear();
	    }
	};

	Acts.prototype.StoreRoaming = function (key, data) {
	    if (this.pluginEnabled) {
	        var roamingSettings = Windows.Storage.ApplicationData.current.roamingSettings;
	        roamingSettings.values[key] = data;
	    }
	};

	Acts.prototype.RemoveRoaming = function (key) {
	    if (this.pluginEnabled) {
	        var roamingSettings = Windows.Storage.ApplicationData.current.roamingSettings;
	        roamingSettings.values.remove(key);
	    }
	};

	Acts.prototype.ClearRoaming = function () {
	    if (this.pluginEnabled) {
	        var roamingSettings = Windows.Storage.ApplicationData.current.roamingSettings;
	        roamingSettings.values.clear();
	    }
	};

	Acts.prototype.SetAdvertisementVisibility = function (AdDivId, idx) {
	    if (this.pluginEnabled && AdDivId !== "") {
	        var adElement = document.getElementById(AdDivId);
	        if (adElement !== undefined && adElement !== null){
	            adElement.style.display = (idx === 0) ? 'block' : 'none';
	        }
	    }
	};
	
	Acts.prototype.RequestAppPurchase = function () {
	    if (this.pluginEnabled) {

	        return (function (self) {
	            if (!self.licenseInformation.isActive || self.licenseInformation.isTrial) {
	                self.currentApp.requestAppPurchaseAsync(false).done(
	                function () {
	                    if (self.licenseInformation.isActive && !self.licenseInformation.isTrial) {
	                        if (self.IsLicenseTestMode) { console.log("Purchased the app"); }
	                        self.runtime.trigger(cr.plugins_.Metro.prototype.cnds.OnPurchaseCompleted, self);
	                    } else {
	                        if (self.IsLicenseTestMode) {console.log("Still in trial mode");}
	                    }
	                },
                    function () {
                        if (self.IsLicenseTestMode) {console.log("Upgrade transaction failed. Still in trial mode");}
                    });
	            }
	        }(this));
	    }
	};

	Acts.prototype.RequestInAppPurchase = function (featureId) {
	    if (this.pluginEnabled) {

	        return function (self) {
	            if (!self.licenseInformation.productLicenses.lookup(featureId).isActive) {
	                self.currentApp.requestProductPurchaseAsync(featureId, false).then(
                        function () {
                            if (self.licenseInformation.productLicenses.lookup(featureId).isActive) {
                                // the in-app purchase was successful
                                // Raise the trigger that purchase was complete
                                if (self.IsLicenseTestMode) { console.log("Purchasing of feature " + featureId + " successful"); }
                                self.runtime.trigger(cr.plugins_.Metro.prototype.cnds.OnPurchaseCompleted, self);
                            } else {
                                // Something went wrong
                                if (self.IsLicenseTestMode) { console.log("Failed purchasing feature " + featureId + " after invoking self.currentApp.requestProductPurchaseAsync"); }
                            }
                        },
                        function (errMsg) {
                            // The in-app purchase was not completed because 
                            // the customer canceled it or there was an error.
                            if (self.IsLicenseTestMode) { console.log("self.currentApp.requestProductPurchaseAsync returned failure" + errMsg); }
                        });
	            }
	            else {
	                // The customer already owns this feature.
	                if (self.IsLicenseTestMode) { console.log(featureId + " License already owned by user"); }
	            }
	        }(this);
	    }
	};
	pluginProto.acts = new Acts();
	
	//////////////////////////////////////
	// Expressions
	function Exps() {};
	
	Exps.prototype.BackgroundMusicUIState = function (ret)	// 'ret' must always be the first parameter - always return the expression's result through it!
	{
	    if (this.pluginEnabled) {
	        ret.set_string((this.GameBackgroundMusicState === true) ? "On" : "Off");
	    }
	};
	Exps.prototype.SoundEffectsUIState = function (ret)	// 'ret' must always be the first parameter - always return the expression's result through it!
	{
	    if (this.pluginEnabled) {
	        ret.set_string((this.GameSoundEffectsState === true) ? "On" : "Off");
	    }
	};
	Exps.prototype.WindowViewState = function (ret)	// 'ret' must always be the first parameter - always return the expression's result through it!
	{
	    if (this.pluginEnabled) {
	        var viewStates = Windows.UI.ViewManagement.ApplicationViewState;

	        switch (this.GameViewState) {
	            case viewStates.snapped:
	                ret.set_string("Snapped");
	                break;
	            case viewStates.filled:
	                ret.set_string("Filled");
	                break;
	            case viewStates.fullScreenLandscape:
	                ret.set_string("FullScreenLandscape");
	                break;
	            case viewStates.fullScreenPortrait:
	                ret.set_string("FullScreenPortrait");
	                break;
	        }
	    }
	};
	
	Exps.prototype.LocalValue = function (ret, key)
	{
	    if (this.pluginEnabled) {
	        var localSettings = Windows.Storage.ApplicationData.current.localSettings;
	        ret.set_string(localSettings.values[key]);
	    } else {
	        ret.set_string("");
	    }
	};

	Exps.prototype.RoamingValue = function (ret, key) {
	    if (this.pluginEnabled) {
	        var roamingSettings = Windows.Storage.ApplicationData.current.roamingSettings;
	        ret.set_string(roamingSettings.values[key]);
	    } else {
	        ret.set_string("");
	    }
	};

    // Deprecated. Use condition IsInAppFeaturePurchased(featureId)
	Exps.prototype.IsInAppPurchaseDone = function () {
	   ret.set_string("No");
	}

	Exps.prototype.AccelerationX = function (ret)
	{
        ret.set_float(this.accX);
	};

	Exps.prototype.AccelerationY = function (ret)
	{
        ret.set_float(this.accY);
	};

	Exps.prototype.AccelerationZ = function (ret)
	{
        ret.set_float(this.accZ);
	};

	Exps.prototype.AngularVelocityX = function (ret) {
        ret.set_float(this.angVelX);
	};

	Exps.prototype.AngularVelocityY = function (ret) {
        ret.set_float(this.angVelY);
	};

	Exps.prototype.AngularVelocityZ = function (ret) {
	    ret.set_float(this.angVelZ);
	};

	Exps.prototype.PitchDegrees = function (ret) {
        ret.set_float(this.pitch);
	};

	Exps.prototype.RollDegrees = function (ret) {
        ret.set_float(this.roll);
	};

	Exps.prototype.YawDegrees = function (ret) {
        ret.set_float(this.yaw);
	};

	Exps.prototype.IlluminanceInLux = function (ret) {
	    ret.set_float(this.illuminanceInLux);
	};
	pluginProto.exps = new Exps();

}());